@@ -1,9 +1,9 @@
-package RevML::Doctype::v0_28 ;
+package RevML::Doctype::v0_29 ;
 
 ##
 ## THIS FILE CREATED AUTOMATICALLY: YOU MAY LOSE ANY EDITS IF YOU MOFIFY IT.
 ##
-## When: Fri Dec 15 07:57:43 2000
+## When: Wed May  8 16:40:27 2002
 ## By:   RevML::Doctype, v0.1, (XML::Doctype, v0.11)
 ##
 
@@ -34,7 +34,7 @@ $doctype = bless( [
         'PATHS' => 6
       },
       undef,
-      '^<name><type>(?:<rev_id>(?:<change_id>)?<digest>|(?:<cvs_info>|<p4_info>|<source_safe_info>|<pvcs_info>)?(?:<branch_id>)?<rev_id>(?:<change_id>)?<time>(?:<mod_time>)?<user_id>(?:<p4_action>|<sourcesafe_action>)?(?:<label>)*(?:<lock>)?(?:<comment>)?(?:<delete>|<move>|(?:<content>|(?:<base_name>)?<base_rev_id><delta>)<digest>))$',
+      '^<name>(?:<type><rev_id>(?:<change_id>)?<digest>|<type>(?:<cvs_info>|<p4_info>|<source_safe_info>|<pvcs_info>)?(?:<branch_id>)?<rev_id>(?:<change_id>)?<time>(?:<mod_time>)?<user_id>(?:<p4_action>|<sourcesafe_action>)?(?:<label>)*(?:<lock>)?(?:<comment>)?(?:<move>|(?:<content>|(?:<base_name>)?<base_rev_id><delta>)<digest>)|(?:<type>)?(?:<cvs_info>|<p4_info>|<source_safe_info>|<pvcs_info>)?(?:<branch_id>)?(?:<rev_id>)?(?:<change_id>)?(?:<time>)?(?:<mod_time>)?(?:<user_id>)?(?:<p4_action>|<sourcesafe_action>)?(?:<label>)*(?:<lock>)?(?:<comment>)?<delete>)$',
       1,
       'rev',
       [
@@ -296,7 +296,7 @@ $doctype = bless( [
       {
         'version' => bless( [
           {},
-          '0.28',
+          '0.29',
           'version',
           undef,
           '#FIXED',
